#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "yopyfunc.h"
#include <usb.h>


void
on_toolbar_quit_clicked                (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


void
on_toolbar_upgrade_clicked             (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
  usb_dev_handle *yopy; /* Handle on USB device */
  GtkWidget *error; /* Error dialogs */
  GtkStatusbar *bar; /* Status Bar */
  GtkProgressBar *progress;
  GtkEntry *entry;

  int num_updates = 0;
  int wait = 0;

  const gchar *filesystem_filename=NULL, *kernel_filename=NULL;
  
  /* Init status and progress bars */
  bar = GTK_STATUSBAR(lookup_widget(GTK_WIDGET(toolbutton), "statusbar"));
  progress = GTK_PROGRESS_BAR(lookup_widget(GTK_WIDGET(toolbutton), "progressbar"));
  

  /* Get string from kernel entry */
  entry = GTK_ENTRY(lookup_widget(GTK_WIDGET(toolbutton), "kernel_path"));
  kernel_filename = gtk_entry_get_text(entry);

  /* Same with filesystem entry */
  entry = GTK_ENTRY(lookup_widget(GTK_WIDGET(toolbutton), "fs_path"));
  filesystem_filename = gtk_entry_get_text(entry);
    
  
  if ( strcmp(filesystem_filename, "") == 0 && strcmp(kernel_filename, "") == 0 )
    {
      error = GTK_WIDGET(create_dialog_no_file_specified());
      gtk_widget_show(error);
      return;
    }

  if ( kernel_filename )
    num_updates += 1;
  if ( filesystem_filename )
    num_updates += 1;
  
  wait = (num_updates > 1) ? 1 : 0;
  
  /* init usb */
  usb_init();
  gtk_statusbar_push(bar, gtk_statusbar_get_context_id(bar, "USB"), "USB initiated.");
  
  gtk_statusbar_push(bar, gtk_statusbar_get_context_id(bar, "USB"), "Trying to open a connection with a Yopy...");
  yopy = open_yopy();
  
  if ( yopy == NULL )
    {
      gtk_statusbar_push(bar, gtk_statusbar_get_context_id(bar, "USB"), "Unable to find a Yopy !");
      error = create_dialog_yopy_not_found();
      gtk_widget_show(error);
      return;
    }
  else
    gtk_statusbar_push(bar, gtk_statusbar_get_context_id(bar, "USB"), "Found a Yopy !");
  
  /* Upgrade kernel */
  if ( kernel_filename )
    {
      gtk_statusbar_push(bar, gtk_statusbar_get_context_id(bar, "UPGRADE"), "Upgrading Kernel...");
      if ( yopy_upgrade(kernel_filename, "YPKERNELUPDATE", yopy, 0, progress) != 1 )
	{
	  return;
	}
    }
  
  /* Upgrade filesystem */
  if ( filesystem_filename )
    {
      gtk_statusbar_push(bar, gtk_statusbar_get_context_id(bar, "UPGRADE"), "Upgrading Filesystem...");
      if ( yopy_upgrade(filesystem_filename, "YPFILESYSUPDATE", yopy, wait, progress) != 1 )
        {
	  return;
        }
    }
  
  
  usb_close(yopy);
  gtk_statusbar_push(bar, gtk_statusbar_get_context_id(bar, "USB"), "Connection with Yopy closed.");
  
}


void
on_toolbar_about_clicked               (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{

}

void OpenKernel( GtkFileSelection *selector, gpointer user_data )
{
  char * filename = ( char * ) gtk_file_selection_get_filename( GTK_FILE_SELECTION(user_data) );
  GtkEntry *ent = lookup_widget(gtk_widget_get_toplevel(selector), "kernel_path");
}


void OpenFileDialog( const char *question, GCallback callback )
{
  GtkFileSelection *FileSelector = ( GtkFileSelection * ) gtk_file_selection_new( question );

  g_signal_connect( G_OBJECT( FileSelector->ok_button ), "clicked", G_CALLBACK( callback ), (gpointer)FileSelector );

  g_signal_connect_swapped( G_OBJECT( FileSelector->ok_button ), "clicked", G_CALLBACK( gtk_widget_destroy ), ( gpointer ) FileSelector );
  g_signal_connect_swapped( G_OBJECT( FileSelector->cancel_button ), "clicked", G_CALLBACK( gtk_widget_destroy ), ( gpointer ) FileSelector );

  gtk_widget_show( GTK_WIDGET( FileSelector ) );
}



void
on_kernel_open_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
  OpenFileDialog( "Select a Kernel file", G_CALLBACK(OpenKernel));
}



void
on_fs_open_button_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_yopy_not_found_close_button_clicked (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *dialog = lookup_widget(GTK_WIDGET(button), "dialog_yopy_not_found");
  gtk_widget_destroy(dialog);
}


void
on_no_file_specified_close_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *dialog = lookup_widget(GTK_WIDGET(button), "dialog_no_file_specified");
  gtk_widget_destroy(dialog);
}




void
on_openfile_cancel_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_openfile_ok_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{

}


